

import java.util.*;
import java.util.concurrent.LinkedBlockingQueue;

/**
 * A class that creates a minimum spanning tree
 * for a given graph
 * The graph is represented by adjacency matrix
 *
 *Lowell Batacan

 * @version 11/21/2015
 */
public class MinimumSpanningTree
{
    // The matrix below represents the graph from the example provided in the lab description
    // The instructor used this hardcoded matrix for testing,
    // your code however must generate the matrix randomly
/*  private int[][] adjMatrix = {{0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
          {0, 0, 0, 3, 1, 0, 0, 0, 0, 0, 2},
            {0, 0, 0, 4, 0, 0, 10, 0, 0, 0, 0},
            {0, 3, 4, 0, 3, 5, 0, 0, 0, 0, 0},
            {0, 1, 0, 3, 0, 2, 0, 0, 0, 0, 0},
            {0, 0, 0, 5, 2, 0, 6, 0, 1, 0, 0},
            {0, 0, 10, 0, 0, 6, 0, 9, 10, 0, 0},
            {0, 0, 0, 0, 0, 0, 9, 0, 0, 0, 0},
            {0, 0, 0, 0, 0, 1, 10, 0, 0, 8, 0},
            {0, 0, 0, 0, 0, 0, 0, 0, 8, 0, 10},
            {0, 2, 0, 0, 0, 0, 0, 0, 0, 10, 0}};
            */
   private int[][] adjMatrix;
    private int[][] minimumSpanningTree;
    private int numberOfNodes;
    final int GRAYED_OUT = -1;
    final int MARKED_FOR_CHECKING = -1;
    final int PERCENT100 = 100;

    public MinimumSpanningTree(int numberOfNodess, int probability) {

        this.numberOfNodes = numberOfNodess;
        this.adjMatrix = new int[this.numberOfNodes+1][this.numberOfNodes+1];
        this.minimumSpanningTree = new int[this.numberOfNodes + 1][this.numberOfNodes + 1];
        Random rand = new Random();
        int randProb;
        int randEdge;
        //populate array
        for (int i = 1; i < this.numberOfNodes; i++) {
            for (int j = 1; j < this.numberOfNodes; j++) {
                this.adjMatrix[i][j] = 0;
                this.adjMatrix[j][i] = 0;
                this.minimumSpanningTree[i][j] = 0;
                this.minimumSpanningTree[j][i] = 0;
            }
        }

        for (int i = 1; i < this.numberOfNodes; i++) {

            for (int j = 1; j < this.numberOfNodes; j++) {
                randProb = rand.nextInt(this.PERCENT100);
                if (i != j) {
                    if (this.adjMatrix[i][j] == 0) {
                        if (randProb > probability) {
                            randEdge = rand.nextInt(this.numberOfNodes) + 1;
                            this.adjMatrix[i][j] = randEdge;
                            this.adjMatrix[j][i] = randEdge;

                        }
                    }
                }
            }

        }

    }


    public boolean isConnected()
    {//for
        //while
        //queue
        //stack -
        boolean connected = false;
       boolean[] visitedArr = new boolean[this.numberOfNodes+1];
        Queue<Integer> visitedQ = new LinkedBlockingQueue<>();
        Queue<Integer> traversalQ = new LinkedBlockingQueue<>();
        visitedQ.add(1);
        traversalQ.add(1);
        visitedArr[1] = true;
       while(!visitedQ.isEmpty())
       {
           int currentR = visitedQ.poll();
           for(int i = 2; i <= this.numberOfNodes; i++)
           {
               if(i != currentR) {
                   if (this.adjMatrix[currentR][i] == 0 && visitedArr[i] == false)
                   {
                       visitedQ.offer(this.adjMatrix[currentR][i]);
                       traversalQ.offer(this.adjMatrix[currentR][i]);
                       visitedArr[i] = true;
                   }
               }
           }
       }


        if(traversalQ.size() == this.numberOfNodes)
        {
            connected = true;
        }
        else
        {
            connected = false;
        }
        return connected;
    }

    public void calculateMinimumSpanningTree()
    {
        int[] key = new int[this.numberOfNodes+1];
        boolean[] msSet = new boolean[this.numberOfNodes+1];

        //initialize arrays
        for(int i =0; i < this.numberOfNodes+1; i++)
        {
            msSet[i] = false;
            key[i] = Integer.MAX_VALUE;
        }
        key[0] = 0;     // Make key 0 so that this vertex is

        for(int i = 0; i < this.numberOfNodes; i++)
        {
            //pick minimum of nodes
            int min = Integer.MAX_VALUE;
            int minIndex = 1;
            for(int j = 1; j <this.numberOfNodes+1; j++)
            {
                if(msSet[j] == false && key[j] <min)
                {
                    min = key[j];
                   minIndex = j;
                }
            }
            int m = minIndex;
            msSet[m] = true;
            for(int j = 1; j < this.numberOfNodes; j++)
            {
                if (this.adjMatrix[i][j]!=0 && msSet[j] == false &&
                        this.adjMatrix[i][j] <  Integer.MAX_VALUE)
                {
                    key[j] = this.adjMatrix[m][j];
                    this.minimumSpanningTree[m][j] = key[j];
                    this.minimumSpanningTree[j][m] = key[j];
                }
            }
        }

    }

    private Element findSmallestInMarkedRows()
    {
        int smallest = this.numberOfNodes;
        int row = 0;
        int col = 0;


        //System.out.println("row= " + row + "; col= " + col + "; smallest= " + smallest);
        return new Element(row, col, smallest);
    }

    public void displayGraph()
    {
        System.out.println("***** GENERATED GRAPH *****");
        displayMatrix(this.adjMatrix);
    }

    public void displayMinimumSpanningTree()
    {
        System.out.println("\n\n***** MINIMUM SPANNING TREE FOR THE ABOVE GRAPH *****");
        displayMatrix(this.minimumSpanningTree);
    }

    private void displayMatrix(int[][] matrix)
    {
        System.out.print("     ");
        for (int c = 1; c <= this.numberOfNodes; c++)
        {
            System.out.printf("[%1$2d]", c);
        }
        System.out.println();
        for (int r = 1; r <= this.numberOfNodes; r++)
        {
            //System.out.print(r + " ");
            System.out.printf("[%1$2d]", r);
            for (int c = 1; c <= this.numberOfNodes; c++)
            {
                if (matrix[r][c] > 0)
                    System.out.printf("%1$4d", matrix[r][c]);
                else
                    System.out.printf("%1$4s", " ");
            }
            System.out.println();
        }
    }

    public static void main(String[] args)
    {
        System.out.println("How many nodes in your graph?");
        Scanner scan = new Scanner(System.in);
        int numberOfNodes = scan.nextInt();
        System.out.println("Probability of edge? (type 70 for 70%)");
        int probability = scan.nextInt();
        MinimumSpanningTree graph = new MinimumSpanningTree(numberOfNodes, probability);
        graph.displayGraph();
        if (graph.isConnected())
        {
            graph.calculateMinimumSpanningTree();
            graph.displayMinimumSpanningTree();
        }
        else
            System.out.println("The graph is not connected, the minimum spanning tree will not be calculated");
    } // end main

    private class Element
    {
        private int row;
        private int column;
        private int value;

        public Element(int row, int column, int value)
        {
            this.row = row;
            this.column = column;
            this.value = value;
        }
    }
}